// SpeakJet.cpp
//
// Copyright (C) 2012 Mike McCauley
// $Id: SpeakJet.cpp,v 1.1 2012/11/26 10:27:42 mikem Exp $

#include <SpeakJet.h>
#include <string.h>
#include <ctype.h>

// Translate an integer into the word required to speak that integer
static char* digitToWord[] = { "zero", "one", "two", "three", "four", "five", "six", "seven", "eight", "nine",
				 "ten", "eleven", "twelve", "thirteen", "fourteen", "fifteen", "sixteen", 
				 "seventeen", "eighteen", "nineteen"};
static char* tensToWord[] = { "zero", "ten", "twenty", "thirty", "forty", "fifty", "sixty", 
			      "seventy", "eighty", "ninety"};

SpeakJet::SpeakJet(uint8_t txPin, uint8_t resetPin, uint8_t readyPin) : SoftwareSerial(0, txPin)
{
    pinMode(txPin, OUTPUT);
    pinMode(_resetPin, OUTPUT);
    pinMode(_readyPin, INPUT);
    
    _resetPin = resetPin;
    _readyPin = readyPin;
}


bool SpeakJet::init()
{
    begin(9600);    
    return reset();
}

bool SpeakJet::reset()
{
    digitalWrite(_resetPin, LOW);
    delay(100);
    digitalWrite(_resetPin, HIGH);
    return true;
}

bool SpeakJet::isReady()
{
    return !digitalRead(_readyPin);
}

void SpeakJet::waitReady()
{
    // Caution: Busy Wait loop
    while (!isReady())
	;
}

void SpeakJet::speakCode(uint8_t code)
{
    waitReady();
    
    // write() is not present before 1.0
    // but print() after 1.0 is buffered.
#if ARDUINO >= 100
    write(code);
#else
    print(code);
#endif
}

void SpeakJet::speakCodes(uint8_t *codes)
{
    while (*codes != EndOfPhrase)
	speakCode(*codes++);
}

bool SpeakJet::speakDictionaryEntry(DictionaryEntry* dict)
{
    uint8_t code;
    uint8_t i = 0;
    do
    {
	// Entry is in PROGMEN and the codes array is also in PROGMEM
	code = pgm_read_byte(pgm_read_word(&dict->codes)+i++);
	speakCode(code);
    } while (code != EndOfPhrase);

    return true;
}

bool SpeakJet::speakWordFromDictionary(const char* word, DictionaryEntry* dict)
{
    // Caution: linear search
    while (pgm_read_word(&dict->word))
    {
	if (strcmp_P(word, (char*)pgm_read_word(&dict->word)) == 0)
	    return speakDictionaryEntry(dict); // Found
	dict++;
    }
    return false; // Not found
}

bool SpeakJet::speakWordsFromDictionary(char* words, DictionaryEntry* dict)
{
    char* delimiters = " \t\n";
    char* token = strtok(words, delimiters);
    while (token)
    {
	speakWordFromDictionary(token, dict);
	token = strtok(NULL, delimiters);
	// Pause between words
	speakCode(Pause1);
    }
    return true;
}

bool SpeakJet::speakWord(const char* word)
{
    extern SpeakJet::DictionaryEntry _dict_standard[];
    return speakWordFromDictionary(word, _dict_standard);
}

bool SpeakJet::speakWords(char* words)
{
    char* delimiters = " \t\n";
    char* token = strtok(words, delimiters);
    while (token)
    {
	speakWord(token);
	token = strtok(NULL, delimiters);
	// Pause between words
	speakCode(Pause1);
    }
    return true;
}

bool SpeakJet::speakDigits(const char* digits)
{
    extern SpeakJet::DictionaryEntry _dict_numeric[];

    while (*digits)
    {
	if (isdigit(*digits))
	    speakWordFromDictionary(digitToWord[*digits - '0'], _dict_numeric);
	else if (*digits == '+')
	    speakWordFromDictionary("plus", _dict_numeric);
	else if (*digits == '-')
	    speakWordFromDictionary("minus", _dict_numeric);
	else if (*digits == '.')
	    speakWordFromDictionary("dot", _dict_numeric);
	// Pause between words
	speakCode(Pause1);
	digits++;
    }
    return true;
}

bool SpeakJet:: speakNumber99(uint8_t n)
{
    extern SpeakJet::DictionaryEntry _dict_numeric[];

    if (n > 99)
	return false;

    if (n <= 19)
	speakWordFromDictionary(digitToWord[n], _dict_numeric);
    else
    {
	uint8_t tens = n / 10;
	speakWordFromDictionary(tensToWord[tens], _dict_numeric);
	if (n % 10) // Dont speak the zero
	    speakWordFromDictionary(digitToWord[n % 10], _dict_numeric);
    }
    return true;
}

bool SpeakJet:: speakNumber999(uint16_t n)
{
    extern SpeakJet::DictionaryEntry _dict_numeric[];

    if (n > 999)
	return false;

    if (n == 0)
	return speakNumber99(0);

    bool sayAnd = (n > 99);
    if (n >= 100)
    {
       	speakWordFromDictionary(digitToWord[n / 100], _dict_numeric);
	speakWordFromDictionary("hundred", _dict_numeric);
	n = n % 100;
    }
    if (n)
    {
	if (sayAnd)
	    speakWordFromDictionary("and", _dict_numeric);
	speakNumber99(n);
    }
}

bool SpeakJet:: speakNumber(uint32_t n)
{
    extern SpeakJet::DictionaryEntry _dict_numeric[];

    if (n == 0)
	return speakNumber99(0);

    bool sayAnd = (n > 999);

    // Billions
    if (n >= 1000000000)
    {
	speakNumber999(n / 1000000000);
	speakWordFromDictionary("billion", _dict_numeric);
	n = n % 1000000000;
    }
    // Millions
    if (n >= 1000000)
    {
	speakNumber999(n / 1000000);
	speakWordFromDictionary("million", _dict_numeric);
	n = n % 1000000;
    }
    // Thousands
    if (n >= 1000)
    {
	speakNumber999(n / 1000);
	speakWordFromDictionary("thousand", _dict_numeric);
	n = n % 1000;
    }
    // Hundreds teens and units
    if (n > 0)
    {
	if (sayAnd && n < 100)
	    speakWordFromDictionary("and", _dict_numeric);
	speakNumber999(n);
    }
    return true;
}

