#!/usr/bin/perl
# -*- mode: Perl -*-
# convertDict.pl
#
# Convert a SpeakJet dictionary in the form of PhraseALator.dic
# into C structures suitable for use in the
# SpeakJet library as a custom dictionary or the standard dicitonary
# Author: Mike McCauley (mikem@open.com.au)
# Copyright (C) 2012 Mike McCauley
# $Id: radiusd,v 1.140 2012/11/22 08:43:25 mikem Exp $

use Getopt::Long;
use strict;

# Translates Phoneme mnemonics to SpeakJet codes
# Phoneme names a standard ones as found in PhraseALator.dic
# Keys are all forced to uppercase, since some dictionary files are casual about case
# Caution: we dont support pitch markers like \NTE2, \NTG2 etc
my %phoneme_to_code = 
(
 '\P0'            => [0],                     # Pause 0ms
 '\P1'            => [1],                     # Pause 100ms
 '\P2'            => [2],                     # Pause 200ms
 '\P3'            => [3],                     # Pause 700ms
 '\P4'            => [4],                     # Pause 30ms
 '\P5'            => [5],                     # Pause 60ms
 '\P6'            => [6],                     # Pause 90ms
 '\FAST'          => [7],                     # Next phoneme at 0.5 speed
 '\SLOW'          => [8],                     # Next phoneme at 1.5 speed
 '\STRESS'        => [14],                    # Next phoneme with some stress
 '\RELAX'         => [15],                    # Next phoneme with relaxation
 '\WAIT'          => [16],                    # Stops and waits for a Start (see manual)
 '\SOFT'          => [18],                    # Next word is soft (not in manual?)
 '\VOLUME'        => [20],                    # Next octet is volume 0 to 127. Default 96
 '\SPEED'         => [21],                    # Next octet is speed 0 to 127. Default 114
 '\PITCH'         => [22],                    # Next octet is pitch in Hz => [to 255
 '\BEND'          => [23],                    # Next octet is frequency bend  to 15. Default is 5
 '\PORTCTR'       => [24],                    # Next octet is port control value. See manual. Default is 7
 '\PORT'          => [25],                    # Next octet is Port Output Value. See manual. Default is 0
 '\REPEAT'        => [26],                    # Next octet is repeat count. 0 to 255
 '\CALLPHRASE'    => [27],                    # Next octet is EEPROM phrase to play and return. See manual.
 '\GOTOPHRASE'    => [28],                    # Next octet is EEPROM phgrase to go to. See manual.
 '\DELAY'         => [29],                    # Next octet is delay in multiples of 10ms. 0 to 255.
 '\RESET'         => [30],                    # Reset Volume Speed], Pitch], Bend to defaults.

 # 32 to 127 reserved

 # 128 to 254 Sound codes
 # Phonemes, standard names
 '\IY'            => [128],                   # 70ms Voiced Long Vowel
 '\IH'            => [129],                   # 70ms Voiced Long Vowel
 '\EY'            => [130],                   # 70ms Voiced Long Vowel
 '\EH'            => [131],                   # 70ms Voiced Long Vowel
 '\AY'            => [132],                   # 70ms Voiced Long Vowel
 '\AX'            => [133],                   # 70ms Voiced Long Vowel
 '\UX'            => [134],                   # 70ms Voiced Long Vowel
 '\OH'            => [135],                   # 70ms Voiced Long Vowel
 '\AW'            => [136],                   # 70ms Voiced Long Vowel
 '\OW'            => [137],                   # 70ms Voiced Long Vowel
 '\UH'            => [138],                   # 70ms Voiced Long Vowel
 '\UW'            => [139],                   # 70ms Voiced Long Vowel
 '\MM'            => [140],                   # 70ms Voiced Nasal
 '\NE'            => [141],                   # 70ms Voiced Nasal
 '\NO'            => [142],                   # 70ms Voiced Nasal
 '\NGE'           => [143],                   # 70ms Voiced Nasal
 '\NGO'           => [144],                   # 70ms Voiced Nasal
 '\LE'            => [145],                   # 70ms Voiced Resonate
 '\LO'            => [146],                   # 70ms Voiced Resonate
 '\WW'            => [147],                   # 70ms Voiced Resonate
 '\RR'            => [148],                   # 70ms Voiced Resonate
 '\IYRR'          => [149],                   # 200ms Voiced R Color Vowel
 '\EYRR'          => [150],                   # 200ms Voiced R Color Vowel
 '\AXRR'          => [151],                   # 190ms Voiced R Color Vowel
 '\AWRR'          => [152],                   # 200ms Voiced R Color Vowel
 '\OWRR'          => [153],                   # 185ms Voiced R Color Vowel
 '\EYIY'          => [154],                   # 165ms Voiced Diphthong
 '\OHIY'          => [155],                   # 200ms Voiced Diphthong
 '\OWIY'          => [156],                   # 225ms Voiced Diphthong
 '\OHIH'          => [157],                   # 185ms Voiced Diphthong
 '\IYEH'          => [158],                   # 170ms Voiced Diphthong
 '\EHLE'          => [159],                   # 140ms Voiced Diphthong
 '\IYUW'          => [160],                   # 180ms Voiced Diphthong
 '\AXUW'          => [161],                   # 170ms Voiced Diphthong
 '\IHWW'          => [162],                   # 170ms Voiced Diphthong
 '\AYWW'          => [163],                   # 200ms Voiced Diphthong
 '\OWWW'          => [164],                   # 131ms Voiced Diphthong
 '\JH'            => [165],                   # 70ms Voiced Affricate
 '\VV'            => [166],                   # 70ms Voiced Fricative
 '\ZZ'            => [167],                   # 70ms Voiced Fricative 
 '\ZH'            => [168],                   # 70ms Voiced Fricative
 '\DH'            => [169],                   # 70ms Voiced Fricative
 '\BE'            => [170],                   # 45ms Voiced Stop
 '\BO'            => [171],                   # 45ms Voiced Stop
 '\EB'            => [172],                   # 10ms Voiced Stop
 '\OB'            => [173],                   # 10ms Voiced Stop
 '\DE'            => [174],                   # 45ms Voiced Stop
 '\DO'            => [175],                   # 45ms Voiced Stop
 '\ED'            => [176],                   # 10ms Voiced Stop
 '\OD'            => [177],                   # 10ms Voiced Stop
 '\GE'            => [178],                   # 55ms Voiced Stop
 '\GO'            => [179],                   # 55ms Voiced Stop
 '\EG'            => [180],                   # 55ms Voiced Stop
 '\OG'            => [181],                   # 55ms Voiced Stop
 '\CH'            => [182],                   # 70ms Voiceless Affricate
 '\HE'            => [183],                   # 70ms Voiceless Fricative
 '\HO'            => [184],                   # 70ms Voiceless Fricative
 '\WH'            => [185],                   # 70ms Voiceless Fricative
 '\FF'            => [186],                   # 70ms Voiceless Fricative
 '\SE'            => [187],                   # 40ms Voiceless Fricative
 '\SO'            => [188],                   # 40ms Voiceless Fricative
 '\SH'            => [189],                   # 50ms Voiceless Fricative
 '\TH'            => [190],                   # 40ms Voiceless Fricative
 '\TT'            => [191],                   # 50ms Voiceless Stop
 '\TU'            => [192],                   # 70ms Voiceless Stop
 '\TS'            => [193],                   # 170ms Voiceless Stop
 '\KE'            => [194],                   # 55ms Voiceless Stop
 '\KO'            => [195],                   # 55ms Voiceless Stop
 '\EK'            => [196],                   # 55ms Voiceless Stop
 '\OK'            => [197],                   # 45ms Voiceless Stop
 '\PE'            => [198],                   # 99ms Voiceless Stop
 '\PO'            => [199],                   # 99ms Voiceless Stop
 # Robot sound
 '\R0'              => [200],                   # 80ms Robot
 '\R1'              => [201],                   # 80ms Robot
 '\R2'              => [202],                   # 80ms Robot
 '\R3'              => [203],                   # 80ms Robot
 '\R4'              => [204],                   # 80ms Robot
 '\R5'              => [205],                   # 80ms Robot
 '\R6'              => [206],                   # 80ms Robot
 '\R7'              => [207],                   # 80ms Robot
 '\R8'              => [208],                   # 80ms Robot
 '\R9'              => [209],                   # 80ms Robot
 # Alarm sound
 '\A0'              => [210],                   # 300ms Alarm
 '\A1'              => [211],                   # 101ms Alarm
 '\A2'              => [212],                   # 102ms Alarm
 '\A3'              => [213],                   # 540ms Alarm
 '\A4'              => [214],                   # 530ms Alarm
 '\A5'              => [215],                   # 500ms Alarm
 '\A6'              => [216],                   # 135ms Alarm
 '\A7'              => [217],                   # 600ms Alarm
 '\A8'              => [218],                   # 300ms Alarm
 '\A9'              => [219],                   # 250ms Alarm
 # Beeps
 '\B0'              => [220],                   # 200ms Beep
 '\B1'              => [221],                   # 270ms Beep
 '\B2'              => [222],                   # 280ms Beep
 '\B3'              => [223],                   # 260ms Beep
 '\B4'              => [224],                   # 300ms Beep
 '\B5'              => [225],                   # 100ms Beep
 '\B6'              => [226],                   # 104ms Beep
 '\B7'              => [227],                   # 100ms Beep
 '\B8'              => [228],                   # 270ms Beep
 '\B9'              => [229],                   # 262ms Beep
 # Biological
 '\C0'              => [230],                   # 160ms Biological
 '\C1'              => [231],                   # 300ms Biological
 '\C2'              => [232],                   # 182ms Biological
 '\C3'              => [233],                   # 120ms Biological
 '\C4'              => [234],                   # 175ms Biological
 '\C5'              => [235],                   # 350ms Biological
 '\C6'              => [236],                   # 160ms Biological
 '\C7'              => [237],                   # 260ms Biological
 '\C8'              => [238],                   # 95ms Biological
 '\C9'              => [239],                   # 75ms Biological
 # DTMF 
 '\DTMF_0'          => [240],                   # DTMF 0 95ms
 '\DTMF_1'          => [241],                   # DTMF 1 95ms
 '\DTMF_2'          => [242],                   # DTMF 2 95ms
 '\DTMF_3'          => [243],                   # DTMF 3 95ms
 '\DTMF_4'          => [244],                   # DTMF 4 95ms
 '\DTMF_5'          => [245],                   # DTMF 5 95ms
 '\DTMF_6'          => [246],                   # DTMF 6 95ms
 '\DTMF_7'          => [247],                   # DTMF 7 95ms
 '\DTMF_8'          => [248],                   # DTMF 8 95ms
 '\DTMF_9'          => [249],                   # DTMF 9 95ms
 '\DTMF_STAR'       => [250],                   # DTMF * 95ms
 '\DTMF_HASH'       => [251],                   # DTMF # 95ms
 # Miscellaneous
 '\M0'              => [252],                   # Sonar ping 125ms
 '\M1'              => [253],                   # Pistol shot 250ms
 '\M2'              => [254],                   # WOW 530ms

 '\ENDOFPHRASE'     => [255],                   # End of phrase marker. Required at end of code arrays

);

# Converts text words into arraycs of codes
my %dictionary;

my @options = 
    (
     'h',                # Help
     'name:s',           # Dictionary name (used to form variable names)
    );

my @original_argv = @ARGV;

&GetOptions(@options) || &usage;

main();

#####################################################################
sub main
{
    while (<ARGV>)
    {
	chomp;
	next if /^\[/; # Skip '[words]' in first line
	parse($_);
    }
    emit();
}

#####################################################################
# Process a single dictioanry line in the format:
# word=\PHONEME \PHONEME \PHONEME ....
# with optional whitespace in various places
# eg:
# airplane=\EYRR \PO \FAST \LE \EYIY \NE
sub parse
{
    my ($line) = @_;

    if ($line =~ /^(\w+)\s*\=\s*(.*)/)
    {
	my ($word, $phonemes) = ($1, $2);
	my @codes; # Assembled array of codes that speaks the word
	my @phonemes = split(/\s+/, $phonemes);
	foreach my $phoneme (@phonemes)
	{
	    if (exists $phoneme_to_code{uc $phoneme})
	    {
		# Defined phoneme
		push(@codes, @{$phoneme_to_code{uc $phoneme}});
	    }
	    elsif ($phoneme =~ /^\\?(\d+)/)
	    {
		# Numeric code, maybe with leading '\'
		push (@codes, $1); 
	    }
	    else
	    {
		# Might be a previously defined word
		# We dont drill down to previously defined words yet
		print STDERR "Warning: no such phoneme: $phoneme for word: $word. Ignoring word\n";
		return;
	    }
	}
	# add EndOfPhrase
	push (@codes, 255);
	@{$dictionary{$word}} = @codes;
#	print "made @codes\n";
    }
}

#####################################################################
sub emit
{
    print "// SpeakJet library dictionary file
// created by:
// $0 @original_argv
// You probably dont want to edit this by hand, but instead alter the .dic file
// and rerun the converter
//
#include <SpeakJet.h>\n";
    my $counter = 0;

    # First emit the word and code definitions
    foreach my $word (sort keys %dictionary)
    {
	print "PROGMEM const char _dict_${main::opt_name}_word_$counter\[\] = \"$word\";\n";
	my $codes = join(', ', @{$dictionary{$word}});
	print "PROGMEM const uint8_t _dict_${main::opt_name}_code_$counter\[\] = { $codes };\n";
	$counter++;
    }

    # And now emit the dictionary array with an entry for each word
    print "PROGMEM SpeakJet::DictionaryEntry _dict_${main::opt_name}[] =\n{\n";
    for (my $i = 0; $i < $counter; $i++)
    {
	print "  { _dict_${main::opt_name}_word_$i, _dict_${main::opt_name}_code_$i },\n";
    }
    print "  { 0, 0 } // flags the end of the dictionary\n};\n";
}

#####################################################################
sub usage
{
    print "usage: $0 <file> ...\n";
}
